# Generated by Django 2.2 on 2019-04-14 17:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0014_auto_20190407_1416'),
    ]

    operations = [
        migrations.RenameField(
            model_name='assignedcontent',
            old_name='persist_token',
            new_name='attached_to_token',
        ),
    ]
