{% extends "registration/base.html" %}
{% load i18n %}
{% block extrahead %}
  {{block.super}}
  <meta http-equiv="refresh" content="3; URL={% url 'auth:account' %}">
{% endblock %}

{% block title %}{{ title }}{% endblock %}
{% block content %}
  <p>{% trans 'Your password was changed.' %}</p>
  <a href="{% url 'auth:account' %}">{% trans 'Click if not redirected' %}</a>
{% endblock %}

{# form errors should not appear here #}
{% block errors %}{% endblock errors%}
