# Generated by Django 2.2.1 on 2019-05-09 15:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0019_contentvariant_valid_feature_for'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='travelprotection',
            name='disallow',
        ),
        migrations.RemoveField(
            model_name='travelprotection',
            name='hashed_secret',
        ),
        migrations.RemoveField(
            model_name='travelprotection',
            name='is_fake',
        ),
        migrations.AddField(
            model_name='travelprotection',
            name='protect_components',
            field=models.ManyToManyField(blank=True, limit_choices_to={'strength__lt': 10}, related_name='travel_protected', to='spider_base.UserComponent'),
        ),
        migrations.AddField(
            model_name='travelprotection',
            name='protect_contents',
            field=models.ManyToManyField(blank=True, related_name='travel_protected', to='spider_base.AssignedContent'),
        ),
        migrations.AlterField(
            model_name='travelprotection',
            name='login_protection',
            field=models.CharField(choices=[('a', 'Hide Protected'), ('c', 'Wipe'), ('d', 'Wipe User')], default='a', max_length=10),
        ),
        migrations.AlterField(
            model_name='travelprotection',
            name='start',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='travelprotection',
            name='stop',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
