{% extends "spider_base/base.html" %}

{% block robots %}none{% endblock %}

{% block extrahead %}
{{block.super}}{% if form %}
{{form.media|safe}}{% endif %}
{% endblock extrahead %}

{% block main_classes %}{{block.super}} w3-padding spkc-content w3-card-4 w3-pale-red{% endblock main_classes %}
