# Generated by Django 2.2.1 on 2019-05-13 15:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0022_auto_20190512_1630'),
    ]

    operations = [
        migrations.AddField(
            model_name='travelprotection',
            name='approved',
            field=models.BooleanField(blank=True, default=False),
        ),
        migrations.AlterField(
            model_name='travelprotection',
            name='login_protection',
            field=models.CharField(choices=[('a', 'Hide'), ('b', 'Hide if triggered'), ('c', 'Disable login'), ('C', 'Disable login if triggered'), ('d', 'Wipe'), ('e', 'Wipe User')], default='a', max_length=1),
        ),
    ]
